<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: MessageFactory.class.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
class MessageFactory extends Factory {
	protected $table = 'message';
	protected $pk_sequence_name = 'message_id_seq'; //PK Sequence name

	protected $type_options = array(
										5 => 'email',
										10 => 'default_schedule',
										20 => 'schedule_amendment',
										30 => 'shift_amendment',
										40 => 'authorization',
										50 => 'request',
										60 => 'job',
										70 => 'job_item',
										80 => 'client',
										100 => 'user' //For notes assigned to users?
									);

	protected $object_name_options = array(
										5 => 'Email', //Email from user to another
										10 => 'Recurring Schedule',
										20 => 'Schedule Amendment',
										30 => 'Shift Amendment',
										40 => 'Authorization',
										50 => 'Request',
										60 => 'Job',
										70 => 'Task',
										80 => 'Client',
										100 => 'Employee' //For notes assigned to users?
									);

	protected $folder_options = array(
										10 => 'Inbox',
										20 => 'Sent'
									);

	protected $status_options = array(
										10 => 'UNREAD',
										20 => 'READ'
									);

	protected $priority_options = array(
										10 => 'LOW',
										50 => 'NORMAL',
										100 => 'HIGH',
										110 => 'URGENT'
									);

	protected $parent_validator_msg = 'Parent is invalid';
	protected $parent_validator_label = 'parent';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $objecttype_validator_options_msg = 'Object Type is invalid';
	protected $objecttype_validator_label = 'object_type';

	protected $object_validator_msg = 'Object ID is invalid';
	protected $object_validator_label = 'object';

	protected $priority_validator_options_msg = 'Invalid Priority';
	protected $priority_validator_label = 'priority';

	protected $statusdate_validator_label = 'status_date';
	protected $ackdate_validator_label = 'ack_date';
	protected $ackby_validator_label = 'ack_by';

	protected $subject_validator_length_msg = 'Invalid Subject length';
	protected $subject_validator_label = 'subject';

	protected $body_validator_length_msg = 'Invalid Body length';
	protected $body_validator_label = 'body';

	function getParent() {
		if ( isset($this->data['parent_id']) ) {
			return $this->data['parent_id'];
		}

		return FALSE;
	}
	function setParent($id) {
		$id = trim($id);

		if ( empty($id) ) {
			$id = 0;
		}

		$mlf = new MessageListFactory();

		if ( $id == 0
				OR $this->Validator->isResultSetWithRows(	$this->parent_validator_label,
															$mlf->getByID($id),
															$this->parent_validator_msg
															) ) {
			$this->data['parent_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getObjectType() {
		if ( isset($this->data['object_type_id']) ) {
			return $this->data['object_type_id'];
		}

		return FALSE;
	}
	function setObjectType($type) {
		$type = trim($type);

		$key = Option::getByValue($type, $this->getOptions('type') );
		if ($key !== FALSE) {
			$type = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->objecttype_validator_label,
											$type,
											$this->objecttype_validator_options_msg,
											$this->getOptions('type')) ) {

			switch ( $type ) {
				case 5:
					$this->obj_handler = new UserListFactory();
					break;
				case 10:
					$this->obj_handler = new DefaultScheduleControlUserListFactory();
					break;
				case 20:
					$this->obj_handler = new ScheduleAmendmentControlListFactory();
					break;
				case 30:
					$this->obj_handler = new ShiftAmendmentListFactory();
					break;
				case 40:
					$this->obj_handler = new AuthorizationListFactory();
					break;
				case 50:
					$this->obj_handler = new RequestListFactory();
					break;
				case 100:
					$this->obj_handler = new UserListFactory();
					break;

			}

			$this->data['object_type_id'] = $type;

			return FALSE;
		}

		return FALSE;
	}

	function getObject() {
		if ( isset($this->data['object_id']) ) {
			return $this->data['object_id'];
		}

		return FALSE;
	}
	function setObject($id) {
		$id = trim($id);

		if ( $this->Validator->isResultSetWithRows(	$this->object_validator_label,
													$this->obj_handler->getByID($id),
													$this->object_validator_msg
													) ) {
			$this->data['object_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getPriority() {
		if ( isset($this->data['priority_id']) ) {
			return $this->data['priority_id'];
		}

		return FALSE;
	}
	function setPriority($priority = NULL) {
		$priority = trim($priority);

		if ( empty($priority) ) {
			$priority = 50;
		}

		$key = Option::getByValue($priority, $this->getOptions('priority') );
		if ($key !== FALSE) {
			$priority = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->priority_validator_label,
											$priority,
											$this->priority_validator_options_msg,
											$this->getOptions('priority')) ) {

			$this->data['priority_id'] = $priority;

			return FALSE;
		}

		return FALSE;
	}

	function getStatus() {
		if ( isset($this->data['status_id']) ) {
			return $this->data['status_id'];
		}

		return FALSE;
	}
	function setStatus($status) {
		$status = trim($status);

		$key = Option::getByValue($status, $this->getOptions('status') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$status,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->setStatusDate();

			$this->data['status_id'] = $status;

			return FALSE;
		}

		return FALSE;
	}

	function getStatusDate() {
		if ( isset($this->data['status_date']) ) {
			return $this->data['status_date'];
		}

		return FALSE;
	}
	function setStatusDate($epoch = NULL) {
		$epoch = trim($epoch);

		if ($epoch == NULL) {
			$epoch = TTDate::getTime();
		}

		if 	(	$this->Validator->isDate(		$this->statusdate_validator_label,
												$epoch,
												$this->date_validator_isdate_msg) ) {

			$this->data['status_date'] = $epoch;

			return TRUE;
		}

		return FALSE;

	}

	function getSubject() {
		if ( isset($this->data['subject']) ) {
			return $this->data['subject'];
		}

		return FALSE;
	}
	function setSubject($text) {
		$text = trim($text);

		if 	(	strlen($text) == 0
				OR
				$this->Validator->isLength(		$this->subject_validator_label,
												$text,
												$this->subject_validator_length_msg,
												2,
												100) ) {

			$this->data['subject'] = htmlentities( $text );

			return TRUE;
		}

		return FALSE;
	}

	function getBody() {
		if ( isset($this->data['body']) ) {
			return $this->data['body'];
		}

		return FALSE;
	}
	function setBody($text) {
		$text = trim($text);

		if 	(	$this->Validator->isLength(		$this->body_validator_label,
												$text,
												$this->body_validator_length_msg,
												5,
												1024) ) {

			$this->data['body'] = htmlentities( $text );

			return TRUE;
		}

		return FALSE;
	}


	function isAck() {
		if ($this->getRequireAck() == TRUE AND $this->getAckDate() == '' ) {
			return FALSE;
		}

		return TRUE;
	}

	function getRequireAck() {
		return $this->fromBool( $this->data['require_ack'] );
	}
	function setRequireAck($bool) {
		$this->data['require_ack'] = $this->toBool($bool);

		return true;
	}

	function getAck() {
		return $this->fromBool( $this->data['ack'] );
	}
	function setAck($bool) {
		$this->data['ack'] = $this->toBool($bool);

		if ( $this->getAck() == TRUE ) {
			$this->setAckDate();
			$this->setAckBy();
		}

		return true;
	}

	function getAckDate() {
		if ( isset($this->data['ack_date']) ) {
			return $this->data['ack_date'];
		}

		return FALSE;
	}
	function setAckDate($epoch = NULL) {
		$epoch = trim($epoch);

		if ($epoch == NULL) {
			$epoch = TTDate::getTime();
		}

		if 	(	$this->Validator->isDate(		$this->ackdate_validator_label,
												$epoch,
												$this->date_validator_isdate_msg) ) {

			$this->data['ack_date'] = $epoch;

			return TRUE;
		}

		return FALSE;

	}

	function getAckBy() {
		if ( isset($this->data['ack_by']) ) {
			return $this->data['ack_by'];
		}

		return FALSE;
	}
	function setAckBy($id = NULL) {
		$id = trim($id);

		if ( empty($id) ) {
			global $current_user;

			if ( is_object($current_user) ) {
				$id = $current_user->getID();
			} else {
				return FALSE;
			}
		}

		$ulf = new UserListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->ackby_validator_label,
													$ulf->getByID($id),
													$this->userby_validator_msg
													) ) {

			$this->data['ack_by'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function postSave() {
		if ( $this->getStatus() == 20 ) {
			global $current_user;

			$this->removeCache( $current_user->getId() );
		}

		return TRUE;
	}

}
?>
