<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: ROEFactory.class.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
class ROEFactory extends Factory {
	protected $table = 'roe';
	protected $pk_sequence_name = 'roe_id_seq'; //PK Sequence name

	var $user_obj = NULL;
	var $pay_stub_entry_account_link_obj = NULL;

	protected $code_options = array(
											'A' 	=> '(A) Shortage of Work',
											'B' 	=> '(B) Strike Or Lockout',
											'C' 	=> '(C) Return to School',
											'D' 	=> '(D) Illness or Injury',
											'E' 	=> '(E) Quit',
											'F' 	=> '(F) Maternity',
											'G' 	=> '(G) Retirement',
											'H' 	=> '(H) Work Sharing',
											'J' 	=> '(J) Apprentice Training',
											'M' 	=> '(M) Dismissal',
											'N' 	=> '(N) Leave of Absence',
											'P' 	=> '(P) Parental',
											'K' 	=> '(K) Other'
									);

	protected $user_validator_msg = 'Invalid User';
	protected $user_validator_label = 'user';

	protected $pay_period_type_validator_options_msg = 'Incorrect pay period type';
	protected $pay_period_type_validator_label = 'pay_period_type_id';

	protected $code_validator_options_msg = 'Incorrect code';
	protected $code_validator_label = 'code_id';

	protected $firstdate_validator_isdate_msg = 'Invalid first date';
	protected $firstdate_validator_label = 'first_date';

	protected $lastdate_validator_isdate_msg = 'Invalid last date';
	protected $lastdate_validator_label = 'last_date';

	protected $pay_period_end_date_validator_isdate_msg = 'Invalid final pay period end date';
	protected $pay_period_end_date_validator_label = 'pay_period_end_date';

	protected $recalldate_validator_isdate_msg = 'Invalid recall date';
	protected $recalldate_validator_label = 'recall_date';

	protected $insurablehours_validator_isfloat_msg = 'Invalid insurable hours';
	protected $insurablehours_validator_label = 'insurable_hours';

	protected $insurableearnings_validator_isfloat_msg = 'Invalid insurable earnings';
	protected $insurableearnings_validator_label = 'insurable_earnings';

	protected $vacationpay_validator_isfloat_msg = 'Invalid vacation pay';
	protected $vacationpay_validator_label = 'vacation_pay';

	protected $serial_validator_islength_msg = 'Invalid serial';
	protected $serial_validator_label = 'serial';

	protected $comments_validator_islength_msg = 'Invalid comments';
	protected $comments_validator_label = 'comments';

	function getUserObject() {
		if ( is_object($this->user_obj) ) {
			return $this->user_obj;
		} else {
			$ulf = new UserListFactory();
			$this->user_obj = $ulf->getById( $this->getUser() )->getCurrent();

			return $this->user_obj;
		}
	}

	function getPayStubEntryAccountLinkObject() {
		if ( is_object($this->pay_stub_entry_account_link_obj) ) {
			return $this->pay_stub_entry_account_link_obj;
		} else {
			$pseallf = new PayStubEntryAccountLinkListFactory();
			$pseallf->getByCompanyID( $this->getUserObject()->getCompany() );
			if ( $pseallf->getRecordCount() > 0 ) {
				$this->pay_stub_entry_account_link_obj = $pseallf->getCurrent();
				return $this->pay_stub_entry_account_link_obj;
			}

			return FALSE;
		}
	}

	function getUser() {
		return $this->data['user_id'];
	}
	function setUser($id) {
		$id = trim($id);

		$ulf = new UserListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->user_validator_label,
															$ulf->getByID($id),
															$this->user_validator_msg
															) ) {
			$this->data['user_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getPayPeriodType() {
		return $this->data['pay_period_type_id'];
	}
	function setPayPeriodType($value) {
		$value = trim($value);

		Debug::Text('Type ID: '. $value, __FILE__, __LINE__, __METHOD__,10);

		$ppsf = new PayPeriodScheduleFactory();

		$key = Option::getByValue($value, $ppsf->getOptions('type') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->pay_period_type_validator_label,
											$value,
											$this->pay_period_type_validator_options_msg,
											$ppsf->getOptions('type')) ) {

			$this->data['pay_period_type_id'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getCode() {
		return $this->data['code_id'];
	}
	function setCode($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('code') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->code_validator_label,
											$value,
											$this->code_validator_options_msg,
											$this->getOptions('code')) ) {

			$this->data['code_id'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getFirstDate() {
		if ( isset($this->data['first_date']) ) {
			return $this->data['first_date'];
		}

		return FALSE;
	}
	function setFirstDate($epoch) {
		$epoch = trim($epoch);

		if 	(	$this->Validator->isDate(		$this->firstdate_validator_label,
												$epoch,
												$this->firstdate_validator_isdate_msg) ) {

			$this->data['first_date'] = $epoch;

			return TRUE;
		}

		return FALSE;

	}

	function getLastDate() {
		if ( isset($this->data['last_date']) ) {
			return $this->data['last_date'];
		}

		return FALSE;
	}
	function setLastDate($epoch) {
		$epoch = trim($epoch);

		//Include the entire day minus one second
		$epoch = TTDate::getBeginDayEpoch( $epoch ) + (86400-120);

		if 	(	$this->Validator->isDate(		$this->lastdate_validator_label,
												$epoch,
												$this->lastdate_validator_isdate_msg) ) {

			$this->data['last_date'] = $epoch;

			return TRUE;
		}

		return FALSE;

	}

	function getPayPeriodEndDate() {
		if ( isset($this->data['pay_period_end_date']) ) {
			return $this->data['pay_period_end_date'];
		}

		return FALSE;
	}
	function setPayPeriodEndDate($epoch) {
		$epoch = trim($epoch);

		if 	(	$this->Validator->isDate(		$this->pay_period_end_date_validator_label,
												$epoch,
												$this->pay_period_end_date_validator_isdate_msg) ) {

			$this->data['pay_period_end_date'] = $epoch;

			return TRUE;
		}

		return FALSE;
	}

	function getRecallDate() {
		if ( isset($this->data['recall_date']) ) {
			return $this->data['recall_date'];
		}

		return FALSE;
	}
	function setRecallDate($epoch) {
		$epoch = trim($epoch);

		if 	(	$this->Validator->isDate(		$this->recalldate_validator_label,
												$epoch,
												$this->recalldate_validator_isdate_msg) ) {

			$this->data['recall_date'] = $epoch;

			return TRUE;
		}

		return FALSE;

	}

	function getInsurableHours() {
		if ( isset($this->data['insurable_hours']) ) {
			return $this->data['insurable_hours'];
		}

		return FALSE;
	}
	function setInsurableHours($value) {
		$value = trim($value);

		if 	(	$this->Validator->isFloat(		$this->insurablehours_validator_label,
												$value,
												$this->insurablehours_validator_isfloat_msg) ) {

			$this->data['insurable_hours'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getInsurableEarnings() {
		if ( isset($this->data['insurable_earnings']) ) {
			return $this->data['insurable_earnings'];
		}

		return FALSE;
	}
	function setInsurableEarnings($value) {
		$value = trim($value);

		if 	(	$this->Validator->isFloat(		$this->insurableearnings_validator_label,
												$value,
												$this->insurableearnings_validator_isfloat_msg) ) {

			$this->data['insurable_earnings'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getVacationPay() {
		if ( isset($this->data['vacation_pay']) ) {
			return $this->data['vacation_pay'];
		}

		return FALSE;
	}
	function setVacationPay($value) {
		$value = trim($value);

		if 	(	$this->Validator->isFloat(		$this->vacationpay_validator_label,
												$value,
												$this->vacationpay_validator_isfloat_msg) ) {

			$this->data['vacation_pay'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getSerial() {
		if ( isset($this->data['serial']) ) {
			return $this->data['serial'];
		}

		return FALSE;
	}
	function setSerial($value) {
		$value = trim($value);

		if 	(	$this->Validator->isLength(		$this->serial_validator_label,
												$value,
												$this->serial_validator_islength_msg,
												9,
												15) ) {

			$this->data['serial'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getComments() {
		if ( isset($this->data['comments']) ) {
			return $this->data['comments'];
		}

		return FALSE;
	}
	function setComments($value) {
		$value = trim($value);

		if 	(	$this->Validator->isLength(		$this->comments_validator_label,
												$value,
												$this->comments_validator_islength_msg,
												0,
												1024) ) {

			$this->data['comments'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getInsurableHoursReportPayPeriods() {
		$report_period_arr = array(
							10  => 53, //'Weekly',
							20 	=> 27, //'Bi-Weekly',
							30  => 25, //'Semi-Monthly',
							40	=> 13, //'Monthly + Advance',
							50  => 13, //'Monthly'

								);

		return $report_period_arr[$this->getPayPeriodType()];
	}

	function getInsurableEarningsReportPayPeriods() {
		$report_period_arr = array(
							10  => 27, //'Weekly',
							20 	=> 14, //'Bi-Weekly',
							30  => 13, //'Semi-Monthly',
							40	=> 7, //'Monthly + Advance',
							50  => 7, //'Monthly'

								);

		return $report_period_arr[$this->getPayPeriodType()];
	}

	function getInsurablePayPeriodStartDate( $pay_periods ) {
		//$pay_periods = $this->getInsurableHoursReportPayPeriods();
		Debug::Text('Pay Periods to Consider: '. $pay_periods, __FILE__, __LINE__, __METHOD__,10);
		Debug::Text('Last Worked Date: '. TTDate::getDate('DATE+TIME', $this->getLastDate() ), __FILE__, __LINE__, __METHOD__,10);


		$pplf = new PayPeriodListFactory();
		$pay_period_obj = $pplf->getByUserIdAndEndDate( $this->getUser(), $this->getLastDate() )->getCurrent();
		Debug::Text('Pay Period ID: '. $pay_period_obj->getId(), __FILE__, __LINE__, __METHOD__,10);

		$pplf->getByPayPeriodScheduleId($pay_period_obj->getPayPeriodSchedule(), NULL, NULL, NULL, array('start_date' => 'desc') );
		$i=1;
		foreach($pplf as $pay_period) {
			//Make sure if there are more pay periods inserted AFTER the last day, we DO NOT include
			//those in the count.
			Debug::Text('Pay Period End Date: '. TTDate::getDate('DATE+TIME', $pay_period->getEndDate() ), __FILE__, __LINE__, __METHOD__,10);

			if ( $this->getLastDate() >= $pay_period->getStartDate() ) {
				Debug::Text($i. '.  Counting ... Pay Period End Date: '. TTDate::getDate('DATE+TIME', $pay_period->getEndDate() ), __FILE__, __LINE__, __METHOD__,10);
				//If there aren't enough pay periods yet, use what we have...
				$start_date = $pay_period->getStartDate();

				if ( $i == $pay_periods ) {
					break;
				}

				$i++;
			}
		}

		Debug::Text('Pay Period Report Start Date: '. TTDate::getDate('DATE+TIME', $start_date), __FILE__, __LINE__, __METHOD__,10);

		return $start_date;
	}

	function getEnableReCalculate() {
		if ( isset($this->recalc) ) {
			return $this->recalc;
		}

		return FALSE;
	}
	function setEnableReCalculate($bool) {
		$this->recalc = $bool;

		return TRUE;
	}

	function reCalculate() {
		//Re-generate final pay stub
		//get current pay period based off their last day of work
		$pplf = new PayPeriodListFactory();
		$pay_period_id = $pplf->getByUserIdAndEndDate( $this->getUser(), $this->getLastDate() )->getCurrent()->getId();
		Debug::Text('Pay Period ID: '. $pay_period_id, __FILE__, __LINE__, __METHOD__,10);

		//Find out if a pay stub is already generated for the pay period we are currently in.
		//If it is, delete it so we can start from fresh
		$pslf = new PayStubListFactory();
		$pslf->getByUserIdAndPayPeriodId( $this->getUser(), $pay_period_id );

		foreach ($pslf as $pay_stub) {
			Debug::Text('Found Pay Stub ID: '. $pay_stub->getId(), __FILE__, __LINE__, __METHOD__,10);
			//Do not delete PAID pay stubs!
			if ( $pay_stub->getStatus() == 10 ) {
				Debug::Text('Last Pay Stub Exists: '. $pay_stub->getId(), __FILE__, __LINE__, __METHOD__,10);
				$pay_stub->setDeleted(TRUE);
				$pay_stub->Save();
			}
		}

		//FIXME: Make sure user isn't already in-active! Otherwise pay stub won't generate.
		Debug::Text('Calculating Pay Stub...', __FILE__, __LINE__, __METHOD__,10);
		$cps = new CalculatePayStub();
		$cps->setUser( $this->getUser() );
		$cps->setPayPeriod( $pay_period_id );
		$cps->calculate();
		Debug::Text('Done Calculating Pay Stub', __FILE__, __LINE__, __METHOD__,10);

		//FIXME: Alert the user if they don't have enough information in TimeTrex to get accurate values.
		//Get insurable hours, earnings, and vacation pay now that the final pay stub is generated

		$ugdlf = new UserGenericDataListFactory();
		$ugdlf->getByCompanyIdAndScriptAndDefault( $this->getUserObject()->getCompany(), $this->getTable() );
		if ( $ugdlf->getRecordCount() > 0 ) {
			Debug::Text('Found Company Form Setup!', __FILE__, __LINE__, __METHOD__,10);
			$ugd_obj = $ugdlf->getCurrent();
			$setup_data = $ugd_obj->getData();
		}
		unset($ugd_obj);

		if ( isset($setup_data) ) {
			//var_dump($setup_data);
			if ( isset($setup_data['insurable_earnings_psea_ids']) ) {
				$insurable_earnings_psea_ids = $setup_data['insurable_earnings_psea_ids'];
			} else {
				//Fall back to Total Gross.
				$insurable_earnings_psea_ids = $this->getPayStubEntryAccountLinkObject()->getTotalGross();
			}
		}


		//Find out the date of how far back we have to go to get insurable values.
		//Insurable Hours
		$insurable_hours_start_date = $this->getInsurablePayPeriodStartDate( $this->getInsurableHoursReportPayPeriods() );

		//Insurable hours dont include stats!Just get total signed in hours.
		//If someone has just a negative bank time balance, those are NOT insurable.
		$udtlf = new UserDateTotalListFactory();
		$worked_total_time = $udtlf->getWorkedTimeSumByUserIDAndStartDateAndEndDate( $this->getUser(), $insurable_hours_start_date, $this->getLastDate() );
		Debug::text('Worked Total Time: '. $worked_total_time, __FILE__, __LINE__, __METHOD__,10);

		$total_hours = Misc::MoneyFormat( TTDate::getHours( $worked_total_time ), FALSE );
		Debug::Text('Total Insurable Hours: '. $total_hours, __FILE__, __LINE__, __METHOD__,10);

		$insurable_earnings_start_date = $this->getInsurablePayPeriodStartDate( $this->getInsurableEarningsReportPayPeriods() );

		$pself = new PayStubEntryListFactory();
		$total_earnings = $pself->getAmountSumByUserIdAndEntryNameIdAndStartDateAndEndDate( $this->getUser(), $insurable_earnings_psea_ids, $insurable_earnings_start_date, $this->getLastDate(), 0 );
		$total_earnings = $total_earnings['amount'];
		//Note, this includes the current pay stub we just generated
		Debug::Text('Total Insurable Earnings: '. $total_earnings, __FILE__, __LINE__, __METHOD__,10);

		//ReSave these
		if ( $this->getId() != '' ) {
			$rlf = new ROEListFactory();
			$rlf->getById( $this->getId() );
			if ( $rlf->getRecordCount() > 0 ) {
				$roe_obj = $rlf->getCurrent();

				$roe_obj->setInsurableHours( $total_hours );
				$roe_obj->setInsurableEarnings( $total_earnings );
				if ( $roe_obj->isValid() ) {
					$roe_obj->Save();
				}
			}
		}

		return TRUE;
	}

	function preSave() {

		//Set User Termination date to Last Day.
		$ulf = new UserListFactory();
		$ulf->getById( $this->getUser() );
		if ( $ulf->getRecordCount() > 0 ) {
			Debug::Text('Setting User Termination Date', __FILE__, __LINE__, __METHOD__,10);

			$user_obj = $ulf->getCurrent();
			$user_obj->setTerminationDate( $this->getLastDate() );
			if ( $user_obj->isValid() ) {
				$user_obj->Save();
			}
		}

		if ( $this->isNew() ) {
			//Create PS amendment releasing all accruals
			PayStubAmendmentFactory::releaseAllAccruals( $this->getUser(), $this->getLastDate() );
			/*
			$psaf = new PayStubAmendmentFactory();
			$psaf->setType( 10 ) ; //Fixed
			$psaf->setUser( $this->getUser() );
			$psaf->setEffectiveDate( $this->getLastDate() );
			$psaf->releaseVacationAccrual();
			if ( $psaf->isValid() ) {
				Debug::Text('Release Vacation Accrual IsValid!!: ', __FILE__, __LINE__, __METHOD__,10);
				$psaf->Save();
			}
			*/
		}

		//Start these off as zero, until we can save this row, and re-calc them after
		//the final pay stub has been generated.
		if ( $this->getInsurableHours() == FALSE ) {
			$this->setInsurableHours( 0 );
		}
		if ( $this->getInsurableEarnings() == FALSE ) {
			$this->setInsurableEarnings( 0 );
		}

		return TRUE;
	}

	function postSave() {
		//Handle dirty work here.
		Debug::Text('ID we just saved: '. $this->getId(), __FILE__, __LINE__, __METHOD__,10);

		if ( $this->getEnableReCalculate() == TRUE ) {
			$this->ReCalculate();
		}

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'ROE', NULL, $this->getTable() );
	}
}
?>
